package com.ruoyi.framework.aspectj;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.tenantdatassource.TenantDataSource;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.reflect.ReflectUtils;
import com.ruoyi.framework.web.service.TokenService;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.plugin.*;
import org.aspectj.lang.annotation.Before;

import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.util.Properties;

/**
 * AOP 拦截SQL处理
 *
 * @author hzh
 */
@Intercepts({
        @Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class})
})
public class SqlInterceptor implements Interceptor {

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        StatementHandler statementHandler = (StatementHandler) invocation.getTarget();
        String sql = statementHandler.getBoundSql().getSql();
        // 修改SQL语句
        String modifiedSql = BeforeSQL(sql);
        if (modifiedSql.equals("error")) {
            // 终止程序
            return AjaxResult.error("当前用户没有数据源信息！");
        } else {
            // 将修改后的SQL语句设置回StatementHandler
            ReflectUtils.setFieldValue(statementHandler.getBoundSql(), "sql", modifiedSql);
            invocation.proceed();
            ReflectUtils.setFieldValue(statementHandler.getBoundSql(), "sql", sql);
            return invocation.proceed();
        }
    }

    // 根据类型设置不同的选择数据源格式
    private String modifySql(String sql) {
        try {
            SecurityUtils.getDatabaseType();
        } catch (Exception e) {
            return "error";
        }
        if (SecurityUtils.getDatabaseType().equals("sqlserver"))
            return "USE `" + SecurityUtils.getDatabaseName() + "`; " + sql;
        if (SecurityUtils.getDatabaseType().equals("dm"))
            return "set schema " + SecurityUtils.getDatabaseName() + "; " + sql;
        if (SecurityUtils.getDatabaseType().equals("mysql"))
            return "USE `" + SecurityUtils.getDatabaseName() + "`; " + sql;
        return "error";
    }

    // 根据类型设置不同的选择数据源格式
    private String BeforeSQL(String sql) {
        try {
            SecurityUtils.getDatabaseType();
        } catch (Exception e) {
            /**
             * 如何当前请求中并不携带该用户的数据源信息
             * 分两种情况：
             * 1.当前用户并没有数据源信息（例如：admin并没有从库数据源信息）
             * 2.当前用户的token失效（理论上在客户端发送请求时会携带token，逻辑代码执行前会首先验证token的有效性）
             * 几乎不会出现第二种情况
             */
            return "error";
        }
        if (SecurityUtils.getDatabaseType().equals("sqlserver"))
            return "USE `" + SecurityUtils.getDatabaseName() + "`; ";
        if (SecurityUtils.getDatabaseType().equals("dm"))
            return "set schema " + SecurityUtils.getDatabaseName() + "; ";
        if (SecurityUtils.getDatabaseType().equals("mysql"))
            return "USE `" + SecurityUtils.getDatabaseName() + "`; ";
        return "error";
    }


    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {
    }
}